/*
   Fractional Wavelet Filter 
   Advanced Version with Lifting Scheme
   Stephan Rein and Martin Reisslein
 */

// compile with:
//gcc -o fractional fractional.c
#include "fractional.h"
#include "EncodePicFlash.h"
#include "DecodePicFlash.h"
#include "FpInvTransformPic.h"
int main(int argc, char **argv)
{
  WelcomeMsg();//print the welcome message
  //unsigned int nbytes=200;//max length of a command line
  size_t nbytes=200;
  char *cmd;    //pointer to command line
  cmd= (char *) malloc ((nbytes + 1)*sizeof(char));
  
  //in case of typed user input:
  while (argc==1) {
    printf("%s",">>");
    getline (&cmd, &nbytes, stdin);
    while (cmd!=NULL) cmd=CheckCmd(cmd);//checks all included commands
  }
  //in case of usage of the program with parameters:
  if (argc>1) { //argc==1 if there is no parameter
    //printf("argc=%i\n",argc);
    int i;cmd[0]=0;
    for (i=1;i<argc;i++) {
      strncat(cmd,argv[i],strlen(argv[i]));//argv[0] is the name of the prg
      if (i<argc-1) strncat(cmd," ",1);//insert a space betw commands
      else strncat(cmd,"\n",1);
    }
    printf("Script usage cmd=%s",cmd);
    while (cmd!=NULL) cmd=CheckCmd(cmd);//checks all included commands  
    return(0);
  }
  return(0);// prevent warning
}


char* CheckCmd(char* cmd){//the follow-up command can be returned
  static INT8 DoLifting=1;//by default do the lifting
  static INT8 ExpFirst=5;// by default the exponent of the wavelet coeff
                         // of the first level = 5
  int NoPixels;
    while (*cmd==32) cmd++;//del the space symb at the beginning
    if (CmpString(cmd,"read")) {INT16 *matrix=ReadPic(GetPara(cmd,1),&NoPixels);
                                    printf("reading image done\n");
                                    WritePic(GetPara(cmd,2),matrix,NoPixels);
				    return GetPara(cmd,3);
    }  
    // ======================================== TRANSFORM:
    else if (CmpString(cmd,"trafo")){     // trafo <PicIn> <PicOut> <level>
      INT16 *matrix=ReadPic(GetPara(cmd,1),&NoPixels);
      printf("NoPixels=%i\n",NoPixels);
      UINT8 level=atoi(GetPara(cmd,3));
      INT16  dest[NoPixels+1];
      dest[0]=Sqrt(NoPixels);//set dimension
      FixWtFlash(matrix,dest,level,DoLifting,ExpFirst); 
  //FpWaveletTransform(matrix,level,ExpFirst);
      WritePic(GetPara(cmd,2),dest,NoPixels);
  //WritePic(GetPara(cmd,2),matrix,NoPixels);
      free(matrix);
      return GetPara(cmd,4); 
    }
    else if (CmpString(cmd,"invtrafo")){     // invtrafo <PicIn> <PicOut> <level>
      INT16 *matrix=ReadPic(GetPara(cmd,1),&NoPixels);
      printf("NoPixels=%i\n",NoPixels);
      UINT8 level=atoi(GetPara(cmd,3));
      //INT16  dest[NoPixels+1];
      //dest[0]=Sqrt(NoPixels);//set dimension
      FpInvWaveletTransform(matrix,level,ExpFirst); 
      //WritePic(GetPara(cmd,2),dest,NoPixels);
      WritePic(GetPara(cmd,2),matrix,NoPixels);
      free(matrix);
      return GetPara(cmd,4); 
    }
    // ========================================  ENCODE:
    else if (CmpString(cmd,"encode")){ // encode <PicIn> <PicOut> <qmin>
      INT16 *matrix=ReadPic(GetPara(cmd,1),&NoPixels);
      printf("NoPixels=%i\n",NoPixels);
      INT8 qmin=atoi(GetPara(cmd,3));
      CompressPicFlash(matrix,qmin,GetPara(cmd,2));//(cmd,2) is <PicOut>  
      free(matrix);
      return GetPara(cmd,4); 
    } 
    // =======================================  DECODE:
    else if (CmpString(cmd,"decode")){// decode 1<BinIn> 2<PicOut> 3<qmin>
      NoPixels=256*256;// should be in the future extracted from the binary input
      printf("NoPixels=%i\n",NoPixels);
      INT16  dest[NoPixels+1];
      dest[0]=Sqrt(NoPixels);//set dimension
      INT8 qmin=atoi(GetPara(cmd,3));
      DecompressPicFlash(dest,qmin,GetPara(cmd,1));//(cmd,1) relates to BinIn
      WritePic(GetPara(cmd,2),dest,NoPixels);//(cmd,2) relates to PicOut
      return GetPara(cmd,4); 
    } 
    // =======================================
    else if (CmpString(cmd,"DoLifting") ){
      DoLifting=atoi(GetPara(cmd,1));
      printf("DoLifting set to %i\n",DoLifting);
      return GetPara(cmd,2);
    }
    else if (CmpString(cmd,"ExpFirst") ){
      ExpFirst=atoi(GetPara(cmd,1));
      printf("ExpFirst set to %i\n",ExpFirst);
      return GetPara(cmd,2);
    }
    else if (CmpString(cmd,"h"))    {PrintHelpMsg();return GetPara(cmd,1);}
    else if (CmpString(cmd,"q"))    exit(0);
    else {printf("Command %s not known.\n",cmd);return GetPara(cmd,1);}
}



char CmpString(char* cmd,char* reference) {
  if (strncmp(reference,cmd,strlen(reference))==0) return 1;
  else return 0;
}
char* GetPara(char* cmd,char no){
  char i;
  for (i=0;i<no;i++){
    
    while ((*cmd!=32)&&(strncmp(cmd,"\n",1) )) cmd++;//move pointer forwards until a space or a "\n" is found
    if (  (*cmd==0)|| !(strncmp(cmd,"\n",1))  )  {
      //printf("error: no parameter\n");
      return NULL;
    }
    while (*cmd==32) cmd++;// del the space between the words
    //printf("%s",cmd);
  }
  
  //printf("parameter:%s",cmd);
  return cmd;
}
 
int GetParaLen(char* cmd){
  //cmd=GetPara(cmd,no);
  
  int i=0;
  //estimate param.length: if cmd != space AND help != Return
  while ((*cmd!=32)&&(strncmp(cmd,"\n",1) )) {cmd++;i++;}
  //printf("parameter length:%i\n",i);
  return i;
}


unsigned int Sqrt(unsigned int n){
    unsigned int a;
    for (a=0;n>=(2*a)+1;n-=(2*a++)+1);
    return a;
}
void PrintHelpMsg(){printf("\
This is the help message.\n\
For a Wavelet Transform:\n\
>>trafo    <PicIn> <PicOut> <level>\n\
>>invtrafo <PicIn> <PicOut> <level>\n\
Switch off lifting scheme (default==1):\n\
>>DoLifting 1\n\
>>ExpFirst 5\n\
>>CODING/DECODING:\n\
>>encode <PicIn> <BinOut> <qmin>\n\
>>decode <BinIn> <PicOut> <qmin>\n");
}

void WelcomeMsg(){printf("\
Wi2l coder -- TU Berlin and Aalborg University\n\
type 'h' for help\
\n");
}
